module Maven

  # $ mvn --version
  # Apache Maven 3.3.9
  # Maven home: /usr/share/maven
  # Java version: 1.8.0_121, vendor: Oracle Corporation
  # Java home: /usr/lib/jvm/java-8-openjdk-amd64/jre
  # Default locale: en_US, platform encoding: UTF-8
  # OS name: "linux", version: "4.9.0-1-amd64", arch: "amd64", family: "unix"
  mvn_cmd_path = File.join(File.expand_path(File.dirname(__FILE__)), '..', '..', '..', 'bin', 'mvn')
  VERSION = `#{mvn_cmd_path} --version`.split("\n")[0].split[2].chomp("\e[m").freeze # => "3.3.9"

  def self.exec( *args )
    if args.member?( '-Dverbose=true' ) || args.member?( '-Dverbose' ) || args.member?( '-X' )
      puts "mvn #{args.join(' ')}"
    end
    old_maven_home = ENV['M2_HOME']
    ENV['M2_HOME'] = home
    system "#{Maven.bin( 'mvn' )} #{args.join( ' ' )}"
  ensure
    ENV['M2_HOME'] = old_maven_home
  end

  def self.home
    @home ||= begin
                dir = File.dirname( File.expand_path( __FILE__ ) )
                File.expand_path( "#{dir}/../../../share/maven" )
              end
  end

  def self.bin( file = nil )
    if file
      File.join( path( 'bin' ), file )
    else
      path( 'bin' )
    end
  end

  def self.lib
    path( 'lib' )
  end

  def self.conf
    path( 'conf' )
  end

  def self.boot
    path( 'boot' )
  end
  
  private

  def self.path( name )
    File.join( home, name )
  end
end
